<?php

class CRM_Loginsecurity_Utils {

  /**
   * Returns the name of the current domain.
   */
  public static function getDomainName() {
    $domain_id = CRM_Core_Config::domainID();

    $domain = \Civi\Api4\Domain::get(FALSE)
      ->addWhere('id', '=', $domain_id)
      ->execute()
      ->first();

    return $domain['name'];
  }

  /**
   * Returns the base domain name/url of the current domain, without https.
   */
  public static function getDomainURL() {
    $url = CIVICRM_UF_BASEURL;
    $url = preg_replace('#https?://#', '', $url);
    return $url;
  }

  /**
   * Check if the OTP was generated in the past 30 minutes.
   */
  public static function isOtpNotExpired($contact_id = NULL): bool {
    if (!$contact_id) {
      $contact_id = CRM_Core_Session::getLoggedInContactID();
    }
    $created_date = CRM_Core_DAO::singleValueQuery('SELECT created_date FROM civicrm_setting WHERE name = %1 AND contact_id = %2 AND created_date >= CURRENT_DATE() - INTERVAL 30 MINUTE', [
      1 => ['loginsecurity_otp', 'String'],
      2 => [$contact_id, 'Positive'],
    ]);
    return (bool) $created_date;
  }

}
