<?php

namespace Civi\Api4\Action\LoginSecurityAuthkey;

use Cose\Algorithms;
use Webauthn\AuthenticatorSelectionCriteria;
use Webauthn\PublicKeyCredentialDescriptor;
use Webauthn\PublicKeyCredentialCreationOptions;
use Webauthn\PublicKeyCredentialParameters;
use Webauthn\PublicKeyCredentialRpEntity;
use Webauthn\PublicKeyCredentialUserEntity;

/**
 * @inheritDoc
 */
class GenerateRegistrationOptions extends \Civi\Api4\Generic\AbstractAction {

  /**
   * Contact ID
   *
   * @var int
   */
  protected $contact_id = '';

  /**
   * @param \Civi\Api4\Generic\Result $result
   *
   * @return void
   * @throws \CRM_Core_Exception
   */
  public function _run(\Civi\Api4\Generic\Result $result) {
    if (empty($this->contact_id)) {
      throw new \CRM_Core_Exception('Missing contact_id');
    }

    // RP Entity i.e. the application
    $rpEntity = PublicKeyCredentialRpEntity::createFromArray([
      'name' => 'My CRM Application',
      'id' => 'standalone20230731.dev501.symbiodev.xyz',
      // 'icon' => null,
    ]);

    // User Entity
    $userEntity = PublicKeyCredentialUserEntity::createFromArray([
      'name' => 'testuser',
      'id' => "1234",
      'displayName' => 'Test User',
      // 'icon' => null,
    ]);

    $challenge = random_bytes(16);

    // Public Key Credential Parameters
    $publicKeyCredentialParametersList = [
      // Recommended minimums by Mozilla and Yubico
      PublicKeyCredentialParameters::create('public-key', Algorithms::COSE_ALGORITHM_ES256),
      PublicKeyCredentialParameters::create('public-key', Algorithms::COSE_ALGORITHM_RS256),
      PublicKeyCredentialParameters::create('public-key', Algorithms::COSE_ALGORITHM_EdDSA),
      // From the php lib example
      #PublicKeyCredentialParameters::create('public-key', Algorithms::COSE_ALGORITHM_ES256K),
      #PublicKeyCredentialParameters::create('public-key', Algorithms::COSE_ALGORITHM_ES384),
      #PublicKeyCredentialParameters::create('public-key', Algorithms::COSE_ALGORITHM_ES512),
      #PublicKeyCredentialParameters::create('public-key', Algorithms::COSE_ALGORITHM_RS384),
      #PublicKeyCredentialParameters::create('public-key', Algorithms::COSE_ALGORITHM_RS512),
      #PublicKeyCredentialParameters::create('public-key', Algorithms::COSE_ALGORITHM_PS256),
      #PublicKeyCredentialParameters::create('public-key', Algorithms::COSE_ALGORITHM_PS384),
      #PublicKeyCredentialParameters::create('public-key', Algorithms::COSE_ALGORITHM_PS512),
      // These make Mozilla crash
      #PublicKeyCredentialParameters::create('public-key', Algorithms::COSE_ALGORITHM_ED256),
      #PublicKeyCredentialParameters::create('public-key', Algorithms::COSE_ALGORITHM_ED512),
    ];

    $publicKeyCredentialCreationOptions = PublicKeyCredentialCreationOptions::create(
      $rpEntity,
      $userEntity,
      $challenge,
      $publicKeyCredentialParametersList,
    );

    $balanceTransactionDetails = ['options' => $publicKeyCredentialCreationOptions];

    $result->exchangeArray($balanceTransactionDetails);
  }

}

